﻿using ELTE.BoardGame.Persistence;
using System;

namespace ELTE.BoardGame.Model
{
    /// <summary>
    /// Connect four játék modell típusa.
    /// </summary>
    public class ConnectFourModel : BoardGameModel
    {
        #region Private constants

        /// <summary>
        /// A nyeréshez szükséges mezők száma.
        /// </summary>
        private const Int32 NumberOfFields = 4;

        #endregion

        #region Constructors

        /// <summary>
        /// Tic-Tac-Toe játék modell példányosítása.
        /// </summary>í
        /// <param name="boardSize">Játéktábla vízszintes, illetve függőleges mérete.</param>
        /// <param name="persistence">Perzisztencia szolgáltatás.</param>
        public ConnectFourModel(Int32 boardSize, IPersistence persistence) 
            : base(boardSize, persistence)
        {
        }

        #endregion
        
        #region Protected methods

        /// <summary>
        /// Játék léptetése.
        /// </summary>
        /// <param name="columnIndex">Oszlop index.</param>
        /// <param name="rowIndex">Sor index.</param>
        protected override void StepGameInternal(Int32 columnIndex, Int32 rowIndex)
        {
            if (this.gameBoard[columnIndex, 0] != PlayerData.NoPlayer) // ellenőrizzük az oszlopot
                throw new InvalidOperationException("Column is full!");

            Int32 actualRowIndex = BoardHeight - 1;
            while (this.gameBoard[columnIndex, actualRowIndex] != PlayerData.NoPlayer)
                actualRowIndex--;

            this.gameBoard[columnIndex, actualRowIndex] = CurrentPlayer; // pozíció rögzítése
            this.OnFieldChanged(columnIndex, actualRowIndex, CurrentPlayer); // jelezzük egy eseménykiváltással, hogy változott a mező
        }

        /// <summary>
        /// Játék ellenőrzése.
        /// </summary>
        protected override void CheckGame() 
        {
            PlayerData won = PlayerData.NoPlayer;

            for (Int32 columnIndex = 0; columnIndex < BoardWidth && won == PlayerData.NoPlayer; columnIndex++)
                for (Int32 rowIndex = 0; rowIndex < BoardHeight && won == PlayerData.NoPlayer; rowIndex++)
                {
                    if (this.gameBoard[columnIndex, rowIndex] == PlayerData.NoPlayer)
                        continue;

                    // ellenőrizzük, valamelyik irányban megvan-e a 4 jel
                     if (this.HaveSameOwner(columnIndex, rowIndex, 0, 1) || this.HaveSameOwner(columnIndex, rowIndex, 1, 0) || this.HaveSameOwner(columnIndex, rowIndex, 1, 1) || this.HaveSameOwner(columnIndex, rowIndex, -1, -1))
                        this.OnGameWon(this.gameBoard[columnIndex, rowIndex]); // esemény kiváltása
                }

            if (StepNumber == BoardHeight * BoardWidth) // döntetlen játék
            {
                this.OnGameOver(); // esemény kiváltása
            }
        }

        /// <summary>
        /// Mezősorozat azonos tulajdonosának ellenőrzése.
        /// </summary>
        /// <param name="startColumn">Kezdő oszlop.</param>
        /// <param name="startRow">Kezdő sor.</param>
        /// <param name="deltaColumn">Oszlopbeli váltás.</param>
        /// <param name="deltaRow">Sorbeli váltás.</param>
        private Boolean HaveSameOwner(Int32 startColumn, Int32 startRow, Int32 deltaColumn, Int32 deltaRow)
        {
            for (Int32 i = 1; i < 4; i++)
            {
                if (startColumn + deltaColumn * i < 0 || startColumn + deltaColumn * i >= BoardHeight ||
                    startRow + deltaRow * i < 0 || startRow + deltaRow * i >= BoardWidth)
                    return false;

                if (this.gameBoard[startColumn, startRow] != this.gameBoard[startColumn + deltaColumn * i, startRow + deltaRow * i])
                    return false;
            }

            return true;
        }

        #endregion
    }
}
