﻿using ELTE.BoardGame.Model;
using System;
using System.Drawing;
using System.Windows.Forms;

namespace ELTE.BoardGame.View
{
    /// <summary>
    /// Táblajáték ablak típusa.
    /// </summary>
    public partial class DrawingForm : BoardGameView
    {
        #region Constructors

        /// <summary>
        /// Tic-Tac-Toe ablak létrehozása.
        /// </summary>
        /// <param name="model">A Tic-Tac-Toe játék.</param>
        public DrawingForm()
        {
            this.InitializeComponent();
        }

        #endregion

        #region BoardGameView properties

        /// <summary>
        /// Tábla szélessége.
        /// </summary>
        public override Int32 BoardWidth { get { return this.panel.Width; } }

        /// <summary>
        /// Tábla magassága.
        /// </summary>
        public override Int32 BoardHeight { get { return this.panel.Height; } }

        #endregion

        #region BoardGameView methods

        /// <summary>
        /// Megjelenített állapot frissítése.
        /// </summary>
        /// <param name="sessionState">A munkafolyamat állapot.</param>
        public override void UpdateViewState(PlayerData[,] sessionState)
        {
            if (sessionState == null)
                return;

            Int32 boardWidth = sessionState.GetLength(0);
            Int32 boardHeight = sessionState.GetLength(1);

            Bitmap bitmap = new Bitmap(this.panel.Width, this.panel.Height); // kép a hatékony kirajzoláshoz

            Graphics graphics = Graphics.FromImage(bitmap);
            graphics.Clear(Color.White); // háttér fehérré festése

            // játéktábla rácsai
            Int32 fieldWidth = this.panel.Width / boardWidth;
            Int32 fieldHeight = this.panel.Height / boardHeight;

            for (Int32 i = 0; i < boardWidth; i++)
                graphics.DrawLine(Pens.Black, 0, i * fieldHeight, this.panel.Width, i * fieldHeight);

            for (Int32 j = 0; j < boardHeight; j++)
                graphics.DrawLine(Pens.Black, j * fieldWidth, 0, j * fieldWidth, this.panel.Height);

            // a mezőtartalmak
            for (Int32 i = 0; i < boardWidth; i++)
                for (Int32 j = 0; j < boardHeight; j++)
                {
                    switch (sessionState[i, j])
                    {
                        case PlayerData.PlayerO:
                            graphics.FillEllipse(Brushes.Yellow, i * fieldWidth + fieldWidth / 10, j * fieldHeight + fieldHeight / 10, 8 * fieldWidth / 10, 8 * fieldHeight / 10);
                            break;
                        case PlayerData.PlayerX:
                            graphics.DrawLine(new Pen(Color.Orange, this.panel.Width / 50), i * fieldWidth + fieldWidth / 10, j * fieldHeight + fieldHeight / 10, i * fieldWidth + 9 * fieldWidth / 10, j * fieldHeight + 9 * fieldHeight / 10);
                            graphics.DrawLine(new Pen(Color.Orange, this.panel.Width / 50), i * fieldWidth + 9 * fieldWidth / 10, j * fieldHeight + fieldHeight / 10, i * fieldWidth + fieldWidth / 10, j * fieldHeight + 9 * fieldHeight / 10);
                            break;
                    }
                }

            this.panel.CreateGraphics().DrawImage(bitmap, 0, 0);
        }

        #endregion

        #region Panel event handlers

        /// <summary>
        /// Panel egérlenyomásának eseménykezelése.
        /// </summary>
        private void Panel_MouseDown(object sender, MouseEventArgs e)
        {
            this.OnBoardClicked(e.X, e.Y);
        }

        #endregion
    }
}
